#!/usr/bin/env python
#
# code to compute refractive index of ice
#

# The MIT License

# Copyright (c) 2016 Ghislain Picard http://lgge.osug.fr/~picard/ice_absorption/

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.


import numpy as np
import numpy.typing npt

#
# based on Picard et al. 2016, in review
# G. Picard, Libois, Q., and Arnaud, L., Refinement of the ice absorption spectrum in the
# visible using radiance profile measurements in Antarctic snow, The Cryosphere Discussion
# doi:10.5194/tc-2016-146, in review, 2016
# data and code available from here: http://lgge.osug.fr/~picard/ice_absorption/


def refice2016(wls: npt.ArrayLike) -> npt.NDArray:
    """return real and imag part of ice refractive index. wavelengths wls are in meter"""
    refice2016_i = ki2016_clean_i / (4 * np.pi) * (wls2016 * 1e-9)

    r2008_r, r2008_i = refice2008(wls)

    wls_nm = wls * 1e9
    r2016_i = np.exp(np.interp(np.log(wls_nm), np.log(wls2016), np.log(refice2016_i)))

    use_2016 = wls_nm <= wls2016[-1]
    merge_i = np.where(use_2016, r2016_i, r2008_i)

    return r2008_r, merge_i


#
# based on Warren and Brandt. 2008
# data are available from here: http://www.atmos.washington.edu/ice_optical_constants/


def refice2008(wls: npt.ArrayLike) -> npt.NDArray:
    """return real and imag part of ice refractive index. wavelengths wls are in meter"""
    return (
        np.interp(wls * 1e9, wl2008, refice2008_r),
        np.exp(np.interp(np.log(wls * 1e9), np.log(wl2008), np.log(refice2008_i))),
    )


#
# based on Warren et al. 1984 but updated in 1995 as described here:
# https://github.com/olemke/arts/blob/master/3rdparty/refice/REFICE.f
# the original ftp site seems not available anymore
# the data are available from here


def refice1995(wls: npt.ArrayLike) -> npt.NDArray:
    """return real and imag part of ice refractive index. wavelengths wls are in meter"""
    return (
        np.interp(wls * 1e9, wl1995, refice1995_r),
        np.exp(np.interp(np.log(wls * 1e9), np.log(wl1995), np.log(refice1995_i))),
    )


# wl in nm
wl2008 = 1000 * np.array(
    [
        1.990e-001,
        2.010e-001,
        2.019e-001,
        2.100e-001,
        2.500e-001,
        3.000e-001,
        3.500e-001,
        3.900e-001,
        4.000e-001,
        4.100e-001,
        4.200e-001,
        4.300e-001,
        4.400e-001,
        4.500e-001,
        4.600e-001,
        4.700e-001,
        4.800e-001,
        4.900e-001,
        5.000e-001,
        5.100e-001,
        5.200e-001,
        5.300e-001,
        5.400e-001,
        5.500e-001,
        5.600e-001,
        5.700e-001,
        5.800e-001,
        5.900e-001,
        6.000e-001,
        6.100e-001,
        6.200e-001,
        6.300e-001,
        6.400e-001,
        6.500e-001,
        6.600e-001,
        6.700e-001,
        6.800e-001,
        6.900e-001,
        7.000e-001,
        7.100e-001,
        7.200e-001,
        7.300e-001,
        7.400e-001,
        7.500e-001,
        7.600e-001,
        7.700e-001,
        7.800e-001,
        7.900e-001,
        8.000e-001,
        8.100e-001,
        8.200e-001,
        8.300e-001,
        8.400e-001,
        8.500e-001,
        8.600e-001,
        8.700e-001,
        8.800e-001,
        8.900e-001,
        9.000e-001,
        9.100e-001,
        9.200e-001,
        9.300e-001,
        9.400e-001,
        9.500e-001,
        9.600e-001,
        9.700e-001,
        9.800e-001,
        9.900e-001,
        1.000e000,
        1.010e000,
        1.020e000,
        1.030e000,
        1.040e000,
        1.050e000,
        1.060e000,
        1.070e000,
        1.080e000,
        1.090e000,
        1.100e000,
        1.110e000,
        1.120e000,
        1.130e000,
        1.140e000,
        1.150e000,
        1.160e000,
        1.170e000,
        1.180e000,
        1.190e000,
        1.200e000,
        1.210e000,
        1.220e000,
        1.230e000,
        1.240e000,
        1.250e000,
        1.260e000,
        1.270e000,
        1.280e000,
        1.290e000,
        1.300e000,
        1.310e000,
        1.320e000,
        1.330e000,
        1.340e000,
        1.350e000,
        1.360e000,
        1.370e000,
        1.380e000,
        1.390e000,
        1.400e000,
        1.410e000,
        1.420e000,
        1.430e000,
        1.440e000,
        1.449e000,
        1.460e000,
        1.471e000,
        1.481e000,
        1.493e000,
        1.504e000,
        1.515e000,
        1.527e000,
        1.538e000,
        1.563e000,
        1.587e000,
        1.613e000,
        1.650e000,
        1.680e000,
        1.700e000,
        1.730e000,
        1.760e000,
        1.800e000,
        1.830e000,
        1.840e000,
        1.850e000,
        1.855e000,
        1.860e000,
        1.870e000,
        1.890e000,
        1.905e000,
        1.923e000,
        1.942e000,
        1.961e000,
        1.980e000,
        2.000e000,
        2.020e000,
        2.041e000,
        2.062e000,
        2.083e000,
        2.105e000,
        2.130e000,
        2.150e000,
        2.170e000,
        2.190e000,
        2.220e000,
        2.240e000,
        2.245e000,
        2.250e000,
        2.260e000,
        2.270e000,
        2.290e000,
        2.310e000,
        2.330e000,
        2.350e000,
        2.370e000,
        2.390e000,
        2.410e000,
        2.430e000,
        2.460e000,
        2.500e000,
        2.520e000,
        2.550e000,
        2.565e000,
        2.580e000,
        2.590e000,
        2.600e000,
        2.620e000,
        2.675e000,
        2.725e000,
        2.778e000,
        2.817e000,
        2.833e000,
        2.849e000,
        2.865e000,
        2.882e000,
        2.899e000,
        2.915e000,
        2.933e000,
        2.950e000,
        2.967e000,
        2.985e000,
        3.003e000,
    ]
)

refice2008_r = np.array(
    [
        1.3943,
        1.3914,
        1.3901,
        1.3801,
        1.3509,
        1.3339,
        1.3249,
        1.3203,
        1.3194,
        1.3185,
        1.3177,
        1.3170,
        1.3163,
        1.3157,
        1.3151,
        1.3145,
        1.3140,
        1.3135,
        1.3130,
        1.3126,
        1.3121,
        1.3117,
        1.3114,
        1.3110,
        1.3106,
        1.3103,
        1.3100,
        1.3097,
        1.3094,
        1.3091,
        1.3088,
        1.3085,
        1.3083,
        1.3080,
        1.3078,
        1.3076,
        1.3073,
        1.3071,
        1.3069,
        1.3067,
        1.3065,
        1.3062,
        1.3060,
        1.3059,
        1.3057,
        1.3055,
        1.3053,
        1.3051,
        1.3049,
        1.3047,
        1.3046,
        1.3044,
        1.3042,
        1.3040,
        1.3039,
        1.3037,
        1.3035,
        1.3033,
        1.3032,
        1.3030,
        1.3028,
        1.3027,
        1.3025,
        1.3023,
        1.3022,
        1.3020,
        1.3019,
        1.3017,
        1.3015,
        1.3014,
        1.3012,
        1.3010,
        1.3009,
        1.3007,
        1.3005,
        1.3003,
        1.3002,
        1.3000,
        1.2998,
        1.2997,
        1.2995,
        1.2993,
        1.2991,
        1.2990,
        1.2988,
        1.2986,
        1.2984,
        1.2982,
        1.2980,
        1.2979,
        1.2977,
        1.2975,
        1.2973,
        1.2971,
        1.2969,
        1.2967,
        1.2965,
        1.2963,
        1.2961,
        1.2959,
        1.2957,
        1.2955,
        1.2953,
        1.2951,
        1.2949,
        1.2946,
        1.2944,
        1.2941,
        1.2939,
        1.2937,
        1.2934,
        1.2931,
        1.2929,
        1.2927,
        1.2924,
        1.2921,
        1.2920,
        1.2918,
        1.2916,
        1.2914,
        1.2912,
        1.2909,
        1.2903,
        1.2897,
        1.2890,
        1.2879,
        1.2870,
        1.2863,
        1.2853,
        1.2843,
        1.2828,
        1.2816,
        1.2811,
        1.2807,
        1.2805,
        1.2802,
        1.2797,
        1.2788,
        1.2780,
        1.2771,
        1.2762,
        1.2756,
        1.2750,
        1.2744,
        1.2736,
        1.2728,
        1.2718,
        1.2707,
        1.2694,
        1.2677,
        1.2663,
        1.2648,
        1.2633,
        1.2609,
        1.2591,
        1.2587,
        1.2582,
        1.2573,
        1.2564,
        1.2545,
        1.2525,
        1.2504,
        1.2482,
        1.2459,
        1.2435,
        1.2409,
        1.2382,
        1.2337,
        1.2270,
        1.2232,
        1.2169,
        1.2135,
        1.2097,
        1.2071,
        1.2043,
        1.1983,
        1.1776,
        1.1507,
        1.1083,
        1.0657,
        1.0453,
        1.0236,
        1.0001,
        0.9747,
        0.9563,
        0.9538,
        0.9678,
        0.9873,
        1.0026,
        1.0180,
        1.0390,
    ]
)

refice2008_i = np.array(
    [
        9.565e-011,
        3.249e-011,
        2.0e-011,
        2.0e-011,
        2.0e-011,
        2.0e-011,
        2.0e-011,
        2.0e-011,
        2.365e-011,
        2.669e-011,
        3.135e-011,
        4.140e-011,
        6.268e-011,
        9.239e-011,
        1.325e-010,
        1.956e-010,
        2.861e-010,
        4.172e-010,
        5.889e-010,
        8.036e-010,
        1.076e-009,
        1.409e-009,
        1.813e-009,
        2.289e-009,
        2.839e-009,
        3.461e-009,
        4.159e-009,
        4.930e-009,
        5.730e-009,
        6.890e-009,
        8.580e-009,
        1.040e-008,
        1.220e-008,
        1.430e-008,
        1.660e-008,
        1.890e-008,
        2.090e-008,
        2.400e-008,
        2.900e-008,
        3.440e-008,
        4.030e-008,
        4.300e-008,
        4.920e-008,
        5.870e-008,
        7.080e-008,
        8.580e-008,
        1.020e-007,
        1.180e-007,
        1.340e-007,
        1.400e-007,
        1.430e-007,
        1.450e-007,
        1.510e-007,
        1.830e-007,
        2.150e-007,
        2.650e-007,
        3.350e-007,
        3.920e-007,
        4.200e-007,
        4.440e-007,
        4.740e-007,
        5.110e-007,
        5.530e-007,
        6.020e-007,
        7.550e-007,
        9.260e-007,
        1.120e-006,
        1.330e-006,
        1.620e-006,
        2.000e-006,
        2.250e-006,
        2.330e-006,
        2.330e-006,
        2.170e-006,
        1.960e-006,
        1.810e-006,
        1.740e-006,
        1.730e-006,
        1.700e-006,
        1.760e-006,
        1.820e-006,
        2.040e-006,
        2.250e-006,
        2.290e-006,
        3.040e-006,
        3.840e-006,
        4.770e-006,
        5.760e-006,
        6.710e-006,
        8.660e-006,
        1.020e-005,
        1.130e-005,
        1.220e-005,
        1.290e-005,
        1.320e-005,
        1.350e-005,
        1.330e-005,
        1.320e-005,
        1.320e-005,
        1.310e-005,
        1.320e-005,
        1.320e-005,
        1.340e-005,
        1.390e-005,
        1.420e-005,
        1.480e-005,
        1.580e-005,
        1.740e-005,
        1.980e-005,
        3.442e-005,
        5.959e-005,
        1.028e-004,
        1.516e-004,
        2.030e-004,
        2.942e-004,
        3.987e-004,
        4.941e-004,
        5.532e-004,
        5.373e-004,
        5.143e-004,
        4.908e-004,
        4.594e-004,
        3.858e-004,
        3.105e-004,
        2.659e-004,
        2.361e-004,
        2.046e-004,
        1.875e-004,
        1.650e-004,
        1.522e-004,
        1.411e-004,
        1.302e-004,
        1.310e-004,
        1.339e-004,
        1.377e-004,
        1.432e-004,
        1.632e-004,
        2.566e-004,
        4.081e-004,
        7.060e-004,
        1.108e-003,
        1.442e-003,
        1.614e-003,
        1.640e-003,
        1.566e-003,
        1.458e-003,
        1.267e-003,
        1.023e-003,
        7.586e-004,
        5.255e-004,
        4.025e-004,
        3.235e-004,
        2.707e-004,
        2.228e-004,
        2.037e-004,
        2.026e-004,
        2.035e-004,
        2.078e-004,
        2.171e-004,
        2.538e-004,
        3.138e-004,
        3.858e-004,
        4.591e-004,
        5.187e-004,
        5.605e-004,
        5.956e-004,
        6.259e-004,
        6.820e-004,
        7.530e-004,
        7.685e-004,
        7.647e-004,
        7.473e-004,
        7.392e-004,
        7.437e-004,
        7.543e-004,
        8.059e-004,
        1.367e-003,
        3.508e-003,
        1.346e-002,
        3.245e-002,
        4.572e-002,
        6.287e-002,
        8.548e-002,
        1.198e-001,
        1.690e-001,
        2.210e-001,
        2.760e-001,
        3.120e-001,
        3.470e-001,
        3.880e-001,
        4.380e-001,
    ]
)


wl1995 = 1000 * np.array(
    [
        4.43000e-02,
        4.51000e-02,
        4.59000e-02,
        4.68000e-02,
        4.77000e-02,
        4.86000e-02,
        4.96000e-02,
        5.06000e-02,
        5.17000e-02,
        5.28000e-02,
        5.39000e-02,
        5.51000e-02,
        5.64000e-02,
        5.77000e-02,
        5.90000e-02,
        6.05000e-02,
        6.20000e-02,
        6.36000e-02,
        6.53000e-02,
        6.70000e-02,
        6.89000e-02,
        7.08000e-02,
        7.29000e-02,
        7.38000e-02,
        7.51000e-02,
        7.75000e-02,
        8.00000e-02,
        8.27000e-02,
        8.55000e-02,
        8.86000e-02,
        9.18000e-02,
        9.30000e-02,
        9.54000e-02,
        9.92000e-02,
        1.03300e-01,
        1.07800e-01,
        1.10000e-01,
        1.12700e-01,
        1.14000e-01,
        1.18100e-01,
        1.21000e-01,
        1.24000e-01,
        1.27200e-01,
        1.29500e-01,
        1.30500e-01,
        1.31900e-01,
        1.33300e-01,
        1.34800e-01,
        1.36200e-01,
        1.37000e-01,
        1.37800e-01,
        1.38700e-01,
        1.39300e-01,
        1.40900e-01,
        1.42500e-01,
        1.43500e-01,
        1.44200e-01,
        1.45000e-01,
        1.45900e-01,
        1.46800e-01,
        1.47600e-01,
        1.48000e-01,
        1.48500e-01,
        1.49400e-01,
        1.51200e-01,
        1.53100e-01,
        1.54000e-01,
        1.55000e-01,
        1.56900e-01,
        1.58000e-01,
        1.58900e-01,
        1.61000e-01,
        1.62700e-01,
        1.65200e-01,
        1.67500e-01,
        1.70000e-01,
        1.72300e-01,
        1.74800e-01,
        1.77100e-01,
        1.79600e-01,
        1.81000e-01,
        1.82000e-01,
        1.83000e-01,
        1.84000e-01,
        1.85000e-01,
        1.88000e-01,
        1.90000e-01,
        1.95000e-01,
        2.00000e-01,
        2.05000e-01,
        2.10000e-01,
        2.15000e-01,
        2.20000e-01,
        2.25000e-01,
        2.30000e-01,
        2.35000e-01,
        2.40000e-01,
        2.45000e-01,
        2.50000e-01,
        2.60000e-01,
        2.70000e-01,
        2.80000e-01,
        2.90000e-01,
        3.00000e-01,
        3.10000e-01,
        3.20000e-01,
        3.30000e-01,
        3.40000e-01,
        3.50000e-01,
        3.60000e-01,
        3.70000e-01,
        3.80000e-01,
        3.90000e-01,
        4.00000e-01,
        4.10000e-01,
        4.20000e-01,
        4.30000e-01,
        4.40000e-01,
        4.50000e-01,
        4.60000e-01,
        4.70000e-01,
        4.80000e-01,
        4.90000e-01,
        5.00000e-01,
        5.10000e-01,
        5.20000e-01,
        5.30000e-01,
        5.40000e-01,
        5.50000e-01,
        5.60000e-01,
        5.70000e-01,
        5.80000e-01,
        5.90000e-01,
        6.00000e-01,
        6.10000e-01,
        6.20000e-01,
        6.30000e-01,
        6.40000e-01,
        6.50000e-01,
        6.60000e-01,
        6.70000e-01,
        6.80000e-01,
        6.90000e-01,
        7.00000e-01,
        7.10000e-01,
        7.20000e-01,
        7.30000e-01,
        7.40000e-01,
        7.50000e-01,
        7.60000e-01,
        7.70000e-01,
        7.80000e-01,
        7.90000e-01,
        8.00000e-01,
        8.10000e-01,
        8.20000e-01,
        8.30000e-01,
        8.40000e-01,
        8.50000e-01,
        8.60000e-01,
        8.70000e-01,
        8.80000e-01,
        8.90000e-01,
        9.00000e-01,
        9.10000e-01,
        9.20000e-01,
        9.30000e-01,
        9.40000e-01,
        9.50000e-01,
        9.60000e-01,
        9.70000e-01,
        9.80000e-01,
        9.90000e-01,
        1.00000,
        1.01000,
        1.02000,
        1.03000,
        1.04000,
        1.05000,
        1.06000,
        1.07000,
        1.08000,
        1.09000,
        1.10000,
        1.11000,
        1.12000,
        1.13000,
        1.14000,
        1.15000,
        1.16000,
        1.17000,
        1.18000,
        1.19000,
        1.20000,
        1.21000,
        1.22000,
        1.23000,
        1.24000,
        1.25000,
        1.26000,
        1.27000,
        1.28000,
        1.29000,
        1.30000,
        1.31000,
        1.32000,
        1.33000,
        1.34000,
        1.35000,
        1.36000,
        1.37000,
        1.38000,
        1.39000,
        1.40000,
        1.41000,
        1.42000,
        1.43000,
        1.44000,
        1.44510,
        1.44930,
        1.45350,
        1.45770,
        1.46200,
        1.46630,
        1.47060,
        1.47490,
        1.47930,
        1.48370,
        1.48810,
        1.49250,
        1.49700,
        1.50150,
        1.50600,
        1.51060,
        1.51520,
        1.51980,
        1.52440,
        1.52910,
        1.53370,
        1.53850,
        1.54320,
        1.54800,
        1.55280,
        1.55760,
        1.56250,
        1.56740,
        1.57230,
        1.57730,
        1.58230,
        1.58730,
        1.59240,
        1.59740,
        1.60260,
        1.60770,
        1.61290,
        1.61810,
        1.62340,
        1.62870,
        1.63400,
        1.63930,
        1.64470,
        1.65020,
        1.65560,
        1.66110,
        1.66670,
        1.67220,
        1.67790,
        1.68350,
        1.68920,
        1.69490,
        1.70070,
        1.70650,
        1.71230,
        1.71820,
        1.72410,
        1.73010,
        1.73610,
        1.74220,
        1.74830,
        1.75440,
        1.76060,
        1.76680,
        1.77300,
        1.77940,
        1.78570,
        1.79210,
        1.79860,
        1.80510,
        1.81160,
        1.81820,
        1.82480,
        1.83150,
        1.83820,
        1.84500,
        1.85190,
        1.85870,
        1.86570,
        1.87270,
        1.87970,
        1.88680,
        1.89390,
        1.90110,
        1.90840,
        1.91570,
        1.92310,
        1.93050,
        1.93800,
        1.94550,
        1.95310,
        1.96080,
        1.96850,
        1.97630,
        1.98410,
        1.99200,
        2.00000,
        2.00800,
        2.01610,
        2.02430,
        2.03250,
        2.04080,
        2.04920,
        2.05760,
        2.06610,
        2.07470,
        2.08330,
        2.09210,
        2.10080,
        2.10970,
        2.11860,
        2.12770,
        2.13680,
        2.14590,
        2.15520,
        2.16450,
        2.17390,
        2.18340,
        2.19300,
        2.20260,
        2.21240,
        2.22220,
        2.23210,
        2.24220,
        2.25230,
        2.26240,
        2.27270,
        2.28310,
        2.29360,
        2.30410,
        2.31480,
        2.32560,
        2.33640,
        2.34740,
        2.35850,
        2.36970,
        2.38100,
        2.39230,
        2.40380,
        2.41550,
        2.42720,
        2.43900,
        2.45100,
        2.46310,
        2.47520,
        2.48760,
        2.50000,
        2.52000,
        2.55000,
        2.56500,
        2.58000,
        2.59000,
        2.60000,
        2.62000,
        2.67500,
        2.72500,
        2.77800,
        2.81700,
        2.83300,
        2.84900,
        2.86500,
        2.88200,
        2.89900,
        2.91500,
        2.93300,
        2.95000,
        2.96700,
        2.98500,
        3.00300,
        3.02100,
        3.04000,
        3.05800,
        3.07700,
        3.09600,
        3.11500,
        3.13500,
        3.15500,
        3.17500,
        3.19500,
        3.21500,
        3.23600,
        3.25700,
        3.27900,
        3.30000,
        3.32200,
        3.34500,
        3.36700,
        3.39000,
        3.41300,
        3.43600,
        3.46000,
        3.48400,
        3.50900,
        3.53400,
        3.55900,
        3.62400,
        3.73200,
        3.77500,
        3.84700,
        3.96900,
        4.09900,
        4.23900,
        4.34800,
        4.38700,
        4.44400,
        4.50500,
        4.54700,
        4.56000,
        4.58000,
        4.71900,
        4.90400,
        5.00000,
        5.10000,
        5.20000,
        5.26300,
        5.40000,
        5.55600,
        5.71400,
        5.74700,
        5.78000,
        5.81400,
        5.84800,
        5.88200,
        6.06100,
        6.13500,
        6.25000,
        6.28900,
        6.32900,
        6.36900,
        6.41000,
        6.45200,
        6.49400,
        6.57900,
        6.66700,
        6.75700,
        6.89700,
        7.04200,
        7.14300,
        7.24600,
        7.35300,
        7.46300,
        7.57600,
        7.69200,
        7.81200,
        7.93700,
        8.06500,
        8.19700,
        8.33300,
        8.47500,
        8.69600,
        8.92900,
        9.09100,
        9.25900,
        9.52400,
        9.80400,
        1.00000e01,
        1.02000e01,
        1.03100e01,
        1.04200e01,
        1.05300e01,
        1.06400e01,
        1.07500e01,
        1.08700e01,
        1.10000e01,
        1.11100e01,
        1.13600e01,
        1.16300e01,
        1.19000e01,
        1.22000e01,
        1.25000e01,
        1.28200e01,
        1.29900e01,
        1.31600e01,
        1.33300e01,
        1.35100e01,
        1.37000e01,
        1.38900e01,
        1.40800e01,
        1.42900e01,
        1.47100e01,
        1.51500e01,
        1.53800e01,
        1.56300e01,
        1.61300e01,
        1.63900e01,
        1.66700e01,
        1.69500e01,
        1.72400e01,
        1.81800e01,
        1.88700e01,
        1.92300e01,
        1.96100e01,
        2.00000e01,
        2.04100e01,
        2.08300e01,
        2.22200e01,
        2.26000e01,
        2.30500e01,
        2.36000e01,
        2.46000e01,
        2.50000e01,
        2.60000e01,
        2.85700e01,
        3.10000e01,
        3.33300e01,
        3.44800e01,
        3.56400e01,
        3.70000e01,
        3.82400e01,
        3.96000e01,
        4.11400e01,
        4.27600e01,
        4.35800e01,
        4.45800e01,
        4.55000e01,
        4.61500e01,
        4.67100e01,
        4.73600e01,
        4.80000e01,
        4.87800e01,
        5.00300e01,
        5.12800e01,
        5.27500e01,
        5.35000e01,
        5.42400e01,
        5.50000e01,
        5.57400e01,
        5.64000e01,
        5.70000e01,
        5.74600e01,
        5.84000e01,
        5.92900e01,
        6.00000e01,
        6.10000e01,
        6.12500e01,
        6.25000e01,
        6.37800e01,
        6.46700e01,
        6.55800e01,
        6.65500e01,
        6.76000e01,
        6.90000e01,
        7.05300e01,
        7.30000e01,
        7.50000e01,
        7.62900e01,
        8.00000e01,
        8.29700e01,
        8.50000e01,
        8.68000e01,
        9.08000e01,
        9.51700e01,
        1.00000e02,
        1.20000e02,
        1.50000e02,
        1.67000e02,
    ]
)

refice1995_r = np.array(
    [
        8.34410e-01,
        8.36760e-01,
        8.37290e-01,
        8.37710e-01,
        8.38270e-01,
        8.40380e-01,
        8.47190e-01,
        8.55220e-01,
        8.60470e-01,
        8.62480e-01,
        8.61570e-01,
        8.60930e-01,
        8.64190e-01,
        8.69160e-01,
        8.77640e-01,
        8.92960e-01,
        9.10410e-01,
        9.30890e-01,
        9.53730e-01,
        9.81880e-01,
        1.02334,
        1.06735,
        1.11197,
        1.13134,
        1.15747,
        1.20045,
        1.23840,
        1.27325,
        1.32157,
        1.38958,
        1.41644,
        1.40906,
        1.40063,
        1.40169,
        1.40934,
        1.40221,
        1.39240,
        1.38424,
        1.38075,
        1.38186,
        1.39634,
        1.40918,
        1.40256,
        1.38013,
        1.36303,
        1.34144,
        1.32377,
        1.30605,
        1.29054,
        1.28890,
        1.28931,
        1.30190,
        1.32025,
        1.36302,
        1.41872,
        1.45834,
        1.49028,
        1.52128,
        1.55376,
        1.57782,
        1.59636,
        1.60652,
        1.61172,
        1.61919,
        1.62522,
        1.63404,
        1.63689,
        1.63833,
        1.63720,
        1.63233,
        1.62222,
        1.58270,
        1.55360,
        1.52040,
        1.49840,
        1.48010,
        1.46710,
        1.45510,
        1.44580,
        1.43700,
        1.43250,
        1.42950,
        1.42680,
        1.42410,
        1.42140,
        1.41430,
        1.41010,
        1.40070,
        1.39360,
        1.38670,
        1.38000,
        1.37610,
        1.37230,
        1.36850,
        1.36480,
        1.36120,
        1.35770,
        1.35420,
        1.35080,
        1.34720,
        1.34370,
        1.34030,
        1.33710,
        1.33390,
        1.33200,
        1.33020,
        1.32840,
        1.32660,
        1.32490,
        1.32380,
        1.32260,
        1.32150,
        1.32040,
        1.31940,
        1.31850,
        1.31775,
        1.31702,
        1.31633,
        1.31569,
        1.31509,
        1.31452,
        1.31399,
        1.31349,
        1.31302,
        1.31257,
        1.31215,
        1.31175,
        1.31136,
        1.31099,
        1.31064,
        1.31031,
        1.30999,
        1.30968,
        1.30938,
        1.30909,
        1.30882,
        1.30855,
        1.30829,
        1.30804,
        1.30780,
        1.30756,
        1.30733,
        1.30710,
        1.30688,
        1.30667,
        1.30646,
        1.30625,
        1.30605,
        1.30585,
        1.30566,
        1.30547,
        1.30528,
        1.30509,
        1.30491,
        1.30473,
        1.30455,
        1.30437,
        1.30419,
        1.30402,
        1.30385,
        1.30367,
        1.30350,
        1.30333,
        1.30316,
        1.30299,
        1.30283,
        1.30266,
        1.30249,
        1.30232,
        1.30216,
        1.30199,
        1.30182,
        1.30166,
        1.30149,
        1.30132,
        1.30116,
        1.30099,
        1.30082,
        1.30065,
        1.30048,
        1.30031,
        1.30014,
        1.29997,
        1.29979,
        1.29962,
        1.29945,
        1.29927,
        1.29909,
        1.29891,
        1.29873,
        1.29855,
        1.29837,
        1.29818,
        1.29800,
        1.29781,
        1.29762,
        1.29743,
        1.29724,
        1.29705,
        1.29686,
        1.29666,
        1.29646,
        1.29626,
        1.29605,
        1.29584,
        1.29563,
        1.29542,
        1.29521,
        1.29499,
        1.29476,
        1.29453,
        1.29430,
        1.29406,
        1.29381,
        1.29355,
        1.29327,
        1.29299,
        1.29272,
        1.29260,
        1.29250,
        1.29240,
        1.29230,
        1.29230,
        1.29220,
        1.29210,
        1.29200,
        1.29190,
        1.29180,
        1.29180,
        1.29170,
        1.29160,
        1.29160,
        1.29150,
        1.29140,
        1.29130,
        1.29120,
        1.29110,
        1.29100,
        1.29090,
        1.29080,
        1.29070,
        1.29060,
        1.29050,
        1.29040,
        1.29030,
        1.29020,
        1.29000,
        1.28990,
        1.28970,
        1.28960,
        1.28950,
        1.28930,
        1.28920,
        1.28900,
        1.28890,
        1.28880,
        1.28860,
        1.28840,
        1.28830,
        1.28810,
        1.28800,
        1.28780,
        1.28760,
        1.28750,
        1.28730,
        1.28710,
        1.28700,
        1.28680,
        1.28660,
        1.28640,
        1.28620,
        1.28600,
        1.28580,
        1.28560,
        1.28540,
        1.28520,
        1.28500,
        1.28470,
        1.28450,
        1.28430,
        1.28410,
        1.28380,
        1.28360,
        1.28340,
        1.28310,
        1.28290,
        1.28260,
        1.28240,
        1.28210,
        1.28190,
        1.28160,
        1.28130,
        1.28100,
        1.28070,
        1.28040,
        1.28010,
        1.27970,
        1.27940,
        1.27900,
        1.27870,
        1.27830,
        1.27790,
        1.27750,
        1.27720,
        1.27690,
        1.27660,
        1.27630,
        1.27600,
        1.27570,
        1.27540,
        1.27510,
        1.27480,
        1.27460,
        1.27430,
        1.27400,
        1.27370,
        1.27340,
        1.27310,
        1.27280,
        1.27240,
        1.27200,
        1.27160,
        1.27120,
        1.27070,
        1.27030,
        1.26980,
        1.26930,
        1.26870,
        1.26810,
        1.26750,
        1.26690,
        1.26620,
        1.26550,
        1.26480,
        1.26410,
        1.26330,
        1.26260,
        1.26180,
        1.26100,
        1.26020,
        1.25930,
        1.25840,
        1.25750,
        1.25650,
        1.25550,
        1.25450,
        1.25340,
        1.25240,
        1.25130,
        1.25020,
        1.24900,
        1.24780,
        1.24650,
        1.24510,
        1.24380,
        1.24240,
        1.24080,
        1.23930,
        1.23770,
        1.23590,
        1.23410,
        1.23220,
        1.23010,
        1.22790,
        1.22580,
        1.22198,
        1.21548,
        1.21184,
        1.20790,
        1.20507,
        1.20209,
        1.19566,
        1.17411,
        1.14734,
        1.10766,
        1.06739,
        1.04762,
        1.02650,
        1.00357,
        9.81970e-01,
        9.65030e-01,
        9.59620e-01,
        9.72690e-01,
        9.91720e-01,
        1.00668,
        1.02186,
        1.04270,
        1.07597,
        1.12954,
        1.21267,
        1.32509,
        1.42599,
        1.49656,
        1.55095,
        1.59988,
        1.63631,
        1.65024,
        1.64278,
        1.62691,
        1.61284,
        1.59245,
        1.57329,
        1.55770,
        1.54129,
        1.52654,
        1.51139,
        1.49725,
        1.48453,
        1.47209,
        1.46125,
        1.45132,
        1.44215,
        1.43366,
        1.41553,
        1.39417,
        1.38732,
        1.37735,
        1.36448,
        1.35414,
        1.34456,
        1.33882,
        1.33807,
        1.33847,
        1.34053,
        1.34287,
        1.34418,
        1.34634,
        1.34422,
        1.33453,
        1.32897,
        1.32333,
        1.31800,
        1.31432,
        1.30623,
        1.29722,
        1.28898,
        1.28730,
        1.28603,
        1.28509,
        1.28535,
        1.28813,
        1.30156,
        1.30901,
        1.31720,
        1.31893,
        1.32039,
        1.32201,
        1.32239,
        1.32149,
        1.32036,
        1.31814,
        1.31705,
        1.31807,
        1.31953,
        1.31933,
        1.31896,
        1.31909,
        1.31796,
        1.31631,
        1.31542,
        1.31540,
        1.31552,
        1.31455,
        1.31193,
        1.30677,
        1.29934,
        1.29253,
        1.28389,
        1.27401,
        1.26724,
        1.25990,
        1.24510,
        1.22241,
        1.19913,
        1.17150,
        1.15528,
        1.13700,
        1.11808,
        1.10134,
        1.09083,
        1.08734,
        1.09254,
        1.10654,
        1.14779,
        1.20202,
        1.25825,
        1.32305,
        1.38574,
        1.44478,
        1.47170,
        1.49619,
        1.51652,
        1.53328,
        1.54900,
        1.56276,
        1.57317,
        1.58028,
        1.57918,
        1.56672,
        1.55869,
        1.55081,
        1.53807,
        1.53296,
        1.53220,
        1.53340,
        1.53289,
        1.51705,
        1.50097,
        1.49681,
        1.49928,
        1.50153,
        1.49856,
        1.49053,
        1.46070,
        1.45182,
        1.44223,
        1.43158,
        1.41385,
        1.40676,
        1.38955,
        1.34894,
        1.31039,
        1.26420,
        1.23656,
        1.21663,
        1.20233,
        1.19640,
        1.19969,
        1.20860,
        1.22173,
        1.24166,
        1.28175,
        1.32784,
        1.38657,
        1.46486,
        1.55323,
        1.60379,
        1.61877,
        1.62963,
        1.65712,
        1.69810,
        1.72065,
        1.74865,
        1.76736,
        1.76476,
        1.75011,
        1.72327,
        1.68490,
        1.62398,
        1.59596,
        1.58514,
        1.59917,
        1.61405,
        1.66625,
        1.70663,
        1.73713,
        1.76860,
        1.80343,
        1.83296,
        1.85682,
        1.87411,
        1.89110,
        1.89918,
        1.90432,
        1.90329,
        1.88744,
        1.87499,
        1.86702,
        1.85361,
        1.84250,
        1.83225,
        1.81914,
        1.82268,
        1.82961,
    ]
)

refice1995_i = np.array(
    [
        1.64000e-01,
        1.73000e-01,
        1.83000e-01,
        1.95000e-01,
        2.08000e-01,
        2.23000e-01,
        2.40000e-01,
        2.50000e-01,
        2.59000e-01,
        2.68000e-01,
        2.79000e-01,
        2.97000e-01,
        3.19000e-01,
        3.40000e-01,
        3.66000e-01,
        3.92000e-01,
        4.16000e-01,
        4.40000e-01,
        4.64000e-01,
        4.92000e-01,
        5.17000e-01,
        5.28000e-01,
        5.33000e-01,
        5.34000e-01,
        5.31000e-01,
        5.24000e-01,
        5.10000e-01,
        5.00000e-01,
        4.99000e-01,
        4.68000e-01,
        3.80000e-01,
        3.60000e-01,
        3.39000e-01,
        3.18000e-01,
        2.91000e-01,
        2.51000e-01,
        2.44000e-01,
        2.39000e-01,
        2.39000e-01,
        2.44000e-01,
        2.47000e-01,
        2.24000e-01,
        1.95000e-01,
        1.74000e-01,
        1.72000e-01,
        1.80000e-01,
        1.94000e-01,
        2.13000e-01,
        2.43000e-01,
        2.71000e-01,
        2.89000e-01,
        3.34000e-01,
        3.44000e-01,
        3.82000e-01,
        4.01000e-01,
        4.06500e-01,
        4.05000e-01,
        3.89000e-01,
        3.77000e-01,
        3.45000e-01,
        3.32000e-01,
        3.15000e-01,
        2.98000e-01,
        2.74000e-01,
        2.28000e-01,
        1.98000e-01,
        1.72000e-01,
        1.56000e-01,
        1.10000e-01,
        8.30000e-02,
        5.80000e-02,
        2.20000e-02,
        1.00000e-02,
        3.00000e-03,
        1.00000e-03,
        3.00000e-04,
        1.00000e-04,
        3.00000e-05,
        1.00000e-05,
        3.00000e-06,
        1.56700e-06,
        9.32500e-07,
        5.39700e-07,
        3.12200e-07,
        1.72500e-07,
        1.00000e-07,
        8.20000e-08,
        5.10000e-08,
        3.81000e-08,
        3.05000e-08,
        2.51000e-08,
        2.18000e-08,
        1.98000e-08,
        1.78000e-08,
        1.62000e-08,
        1.50000e-08,
        1.43000e-08,
        1.37000e-08,
        1.33000e-08,
        1.32000e-08,
        1.30000e-08,
        1.26000e-08,
        1.18000e-08,
        1.10000e-08,
        9.28000e-09,
        8.25000e-09,
        7.65000e-09,
        7.00000e-09,
        6.15000e-09,
        5.10000e-09,
        4.13000e-09,
        3.43000e-09,
        3.12000e-09,
        2.82000e-09,
        2.51000e-09,
        2.26000e-09,
        2.08000e-09,
        1.91000e-09,
        1.54000e-09,
        1.53000e-09,
        1.55000e-09,
        1.64000e-09,
        1.78000e-09,
        1.91000e-09,
        2.14000e-09,
        2.26000e-09,
        2.54000e-09,
        2.93000e-09,
        3.11000e-09,
        3.29000e-09,
        3.52000e-09,
        4.04000e-09,
        4.88000e-09,
        5.73000e-09,
        6.89000e-09,
        8.58000e-09,
        1.04000e-08,
        1.22000e-08,
        1.43000e-08,
        1.66000e-08,
        1.89000e-08,
        2.09000e-08,
        2.40000e-08,
        2.90000e-08,
        3.44000e-08,
        4.03000e-08,
        4.30000e-08,
        4.92000e-08,
        5.87000e-08,
        7.08000e-08,
        8.58000e-08,
        1.02000e-07,
        1.18000e-07,
        1.34000e-07,
        1.40000e-07,
        1.43000e-07,
        1.45000e-07,
        1.51000e-07,
        1.83000e-07,
        2.15000e-07,
        2.65000e-07,
        3.35000e-07,
        3.92000e-07,
        4.20000e-07,
        4.44000e-07,
        4.74000e-07,
        5.11000e-07,
        5.53000e-07,
        6.02000e-07,
        7.55000e-07,
        9.26000e-07,
        1.12000e-06,
        1.33000e-06,
        1.62000e-06,
        2.00000e-06,
        2.25000e-06,
        2.33000e-06,
        2.33000e-06,
        2.17000e-06,
        1.96000e-06,
        1.81000e-06,
        1.74000e-06,
        1.73000e-06,
        1.70000e-06,
        1.76000e-06,
        1.82000e-06,
        2.04000e-06,
        2.25000e-06,
        2.29000e-06,
        3.04000e-06,
        3.84000e-06,
        4.77000e-06,
        5.76000e-06,
        6.71000e-06,
        8.66000e-06,
        1.02000e-05,
        1.13000e-05,
        1.22000e-05,
        1.29000e-05,
        1.32000e-05,
        1.35000e-05,
        1.33000e-05,
        1.32000e-05,
        1.32000e-05,
        1.31000e-05,
        1.32000e-05,
        1.32000e-05,
        1.34000e-05,
        1.39000e-05,
        1.42000e-05,
        1.48000e-05,
        1.58000e-05,
        1.74000e-05,
        1.98000e-05,
        2.50000e-05,
        5.40000e-05,
        9.00000e-05,
        1.30000e-04,
        1.60000e-04,
        1.89000e-04,
        2.20000e-04,
        2.58000e-04,
        2.99000e-04,
        3.42000e-04,
        3.86000e-04,
        4.33000e-04,
        4.71000e-04,
        5.03000e-04,
        5.27000e-04,
        5.34000e-04,
        5.38000e-04,
        5.33000e-04,
        5.27000e-04,
        5.17000e-04,
        5.09000e-04,
        4.98000e-04,
        4.87000e-04,
        4.75000e-04,
        4.62000e-04,
        4.51000e-04,
        4.37000e-04,
        4.25000e-04,
        4.08000e-04,
        3.92000e-04,
        3.75000e-04,
        3.58000e-04,
        3.42000e-04,
        3.24000e-04,
        3.11000e-04,
        2.97000e-04,
        2.84000e-04,
        2.74000e-04,
        2.66000e-04,
        2.59000e-04,
        2.53000e-04,
        2.49000e-04,
        2.47000e-04,
        2.44000e-04,
        2.40000e-04,
        2.35000e-04,
        2.29000e-04,
        2.23000e-04,
        2.16000e-04,
        2.11000e-04,
        2.05000e-04,
        1.97000e-04,
        1.92000e-04,
        1.87000e-04,
        1.82000e-04,
        1.78000e-04,
        1.72000e-04,
        1.67000e-04,
        1.63000e-04,
        1.59000e-04,
        1.54000e-04,
        1.52000e-04,
        1.49000e-04,
        1.48000e-04,
        1.45000e-04,
        1.42000e-04,
        1.40000e-04,
        1.39000e-04,
        1.38000e-04,
        1.37000e-04,
        1.35000e-04,
        1.33000e-04,
        1.31000e-04,
        1.30000e-04,
        1.28000e-04,
        1.26000e-04,
        1.25000e-04,
        1.23000e-04,
        1.23000e-04,
        1.26000e-04,
        1.26000e-04,
        1.33000e-04,
        1.42000e-04,
        1.56000e-04,
        1.87000e-04,
        2.33000e-04,
        2.98000e-04,
        4.00000e-04,
        4.84000e-04,
        5.53000e-04,
        6.82000e-04,
        8.38000e-04,
        9.91000e-04,
        1.16000e-03,
        1.31000e-03,
        1.42000e-03,
        1.52000e-03,
        1.59000e-03,
        1.63000e-03,
        1.64000e-03,
        1.64000e-03,
        1.63000e-03,
        1.59000e-03,
        1.55000e-03,
        1.52000e-03,
        1.45000e-03,
        1.40000e-03,
        1.32000e-03,
        1.23000e-03,
        1.13000e-03,
        1.02000e-03,
        9.30000e-04,
        8.20000e-04,
        7.19000e-04,
        6.28000e-04,
        5.50000e-04,
        4.85000e-04,
        4.28000e-04,
        3.83000e-04,
        3.48000e-04,
        3.17000e-04,
        2.93000e-04,
        2.71000e-04,
        2.54000e-04,
        2.38000e-04,
        2.27000e-04,
        2.17000e-04,
        2.12000e-04,
        2.10000e-04,
        2.16000e-04,
        2.28000e-04,
        2.46000e-04,
        2.72000e-04,
        3.03000e-04,
        3.40000e-04,
        3.81000e-04,
        4.20000e-04,
        4.61000e-04,
        4.97000e-04,
        5.27000e-04,
        5.54000e-04,
        5.77000e-04,
        5.99000e-04,
        6.21000e-04,
        6.40000e-04,
        6.61000e-04,
        6.91000e-04,
        7.20000e-04,
        7.44000e-04,
        7.72000e-04,
        8.04000e-04,
        8.25500e-04,
        8.57800e-04,
        8.73900e-04,
        8.90000e-04,
        9.30000e-04,
        1.01000e-03,
        1.35000e-03,
        3.42000e-03,
        7.92000e-03,
        2.00000e-02,
        3.80000e-02,
        5.20000e-02,
        6.80000e-02,
        9.23000e-02,
        1.27000e-01,
        1.69000e-01,
        2.21000e-01,
        2.76000e-01,
        3.12000e-01,
        3.47000e-01,
        3.88000e-01,
        4.38000e-01,
        4.93000e-01,
        5.54000e-01,
        6.12000e-01,
        6.25000e-01,
        5.93000e-01,
        5.39000e-01,
        4.91000e-01,
        4.38000e-01,
        3.72000e-01,
        3.00000e-01,
        2.38000e-01,
        1.93000e-01,
        1.58000e-01,
        1.21000e-01,
        1.03000e-01,
        8.36000e-02,
        6.68000e-02,
        5.40000e-02,
        4.22000e-02,
        3.42000e-02,
        2.74000e-02,
        2.20000e-02,
        1.86000e-02,
        1.52000e-02,
        1.26000e-02,
        1.06000e-02,
        8.02000e-03,
        6.85000e-03,
        6.60000e-03,
        6.96000e-03,
        9.16000e-03,
        1.11000e-02,
        1.45000e-02,
        2.00000e-02,
        2.30000e-02,
        2.60000e-02,
        2.90000e-02,
        2.93000e-02,
        3.00000e-02,
        2.85000e-02,
        1.73000e-02,
        1.29000e-02,
        1.20000e-02,
        1.25000e-02,
        1.34000e-02,
        1.40000e-02,
        1.75000e-02,
        2.40000e-02,
        3.50000e-02,
        3.80000e-02,
        4.20000e-02,
        4.60000e-02,
        5.20000e-02,
        5.70000e-02,
        6.90000e-02,
        7.00000e-02,
        6.70000e-02,
        6.50000e-02,
        6.40000e-02,
        6.20000e-02,
        5.90000e-02,
        5.70000e-02,
        5.60000e-02,
        5.50000e-02,
        5.70000e-02,
        5.80000e-02,
        5.70000e-02,
        5.50000e-02,
        5.50000e-02,
        5.40000e-02,
        5.20000e-02,
        5.20000e-02,
        5.20000e-02,
        5.20000e-02,
        5.00000e-02,
        4.70000e-02,
        4.30000e-02,
        3.90000e-02,
        3.70000e-02,
        3.90000e-02,
        4.00000e-02,
        4.20000e-02,
        4.40000e-02,
        4.50000e-02,
        4.60000e-02,
        4.70000e-02,
        5.10000e-02,
        6.50000e-02,
        7.50000e-02,
        8.80000e-02,
        1.08000e-01,
        1.34000e-01,
        1.68000e-01,
        2.04000e-01,
        2.48000e-01,
        2.80000e-01,
        3.41000e-01,
        3.79000e-01,
        4.09000e-01,
        4.22000e-01,
        4.22000e-01,
        4.03000e-01,
        3.89000e-01,
        3.74000e-01,
        3.54000e-01,
        3.35000e-01,
        3.15000e-01,
        2.94000e-01,
        2.71000e-01,
        2.46000e-01,
        1.98000e-01,
        1.64000e-01,
        1.52000e-01,
        1.42000e-01,
        1.28000e-01,
        1.25000e-01,
        1.23000e-01,
        1.16000e-01,
        1.07000e-01,
        7.90000e-02,
        7.20000e-02,
        7.60000e-02,
        7.50000e-02,
        6.70000e-02,
        5.50000e-02,
        4.50000e-02,
        2.90000e-02,
        2.75000e-02,
        2.70000e-02,
        2.73000e-02,
        2.89000e-02,
        3.00000e-02,
        3.40000e-02,
        5.30000e-02,
        7.55000e-02,
        1.06000e-01,
        1.35000e-01,
        1.76100e-01,
        2.22900e-01,
        2.74600e-01,
        3.28000e-01,
        3.90600e-01,
        4.64200e-01,
        5.24700e-01,
        5.73100e-01,
        6.36200e-01,
        6.83900e-01,
        7.09100e-01,
        6.79000e-01,
        6.25000e-01,
        5.65400e-01,
        5.43300e-01,
        5.29200e-01,
        5.07000e-01,
        4.88300e-01,
        4.70700e-01,
        4.20300e-01,
        3.77100e-01,
        3.37600e-01,
        3.05600e-01,
        2.83500e-01,
        3.17000e-01,
        3.51700e-01,
        3.90200e-01,
        4.50900e-01,
        4.67100e-01,
        4.77900e-01,
        4.89000e-01,
        4.89900e-01,
        4.87300e-01,
        4.76600e-01,
        4.50800e-01,
        4.19300e-01,
        3.88000e-01,
        3.43300e-01,
        3.11800e-01,
        2.93500e-01,
        2.35000e-01,
        1.98100e-01,
        1.86500e-01,
        1.77100e-01,
        1.62000e-01,
        1.49000e-01,
        1.39000e-01,
        1.20000e-01,
        9.62000e-02,
        8.30000e-02,
    ]
)


wls2016 = np.arange(320.0, 605.0, 5)

# ice absorption obtain by Picard et la.
ki2016_clean_i = np.array(
    [
        3.294254536192534055e-02,
        3.076769476468334957e-02,
        3.011907051488688786e-02,
        2.893488343802427809e-02,
        2.762179391847458670e-02,
        2.601868266109514868e-02,
        2.510054695844085929e-02,
        2.400262236177611724e-02,
        2.317015173847364676e-02,
        2.256078972698605245e-02,
        2.200148469997523573e-02,
        2.144447888164110333e-02,
        2.123184513553833669e-02,
        2.049138370588751007e-02,
        2.042711755818010094e-02,
        1.960181068698919893e-02,
        1.971384489706732440e-02,
        1.880622912629118654e-02,
        1.792598305524367372e-02,
        1.730682287339309691e-02,
        1.724723932081185726e-02,
        1.734140548179877520e-02,
        1.745388470629162872e-02,
        1.789922545195348283e-02,
        1.820484920895810640e-02,
        1.867058086944543491e-02,
        1.925604777510541918e-02,
        1.984733608033510677e-02,
        2.054244786783458385e-02,
        2.134832555231510123e-02,
        2.246472424317939171e-02,
        2.360184807719198363e-02,
        2.478905138575037206e-02,
        2.614137310789671140e-02,
        2.767465540928885567e-02,
        2.943161559730156709e-02,
        3.130691172450063287e-02,
        3.345979640410464556e-02,
        3.589531371661214970e-02,
        3.819224277052086364e-02,
        4.098488407883403573e-02,
        4.388446405361826064e-02,
        4.708083395543986860e-02,
        5.067129262305845178e-02,
        5.461812971470113298e-02,
        5.915958545150271780e-02,
        6.399197513073895394e-02,
        6.936337992698332622e-02,
        7.508639336855899016e-02,
        8.106493224269085973e-02,
        8.729770962108315824e-02,
        9.401733067082994333e-02,
        1.007896720807269081e-01,
        1.083179776540570127e-01,
        1.166354851163920320e-01,
        1.260482198488585281e-01,
        1.359718887014898236e-01,
    ]
)
